import React, { Component } from 'react';
import PropTypes$1, { PropTypes } from 'prop-types';
import Modal from 'react-modal';

var triggerCustomEvent = function triggerCustomEvent(eventName) {
  if (typeof window !== 'undefined') {
    window.dispatchEvent(new Event(eventName));
  }
};

var unique = function unique() {
  var array = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

  return array.filter(function (val, index, arr) {
    return arr.indexOf(val) === index;
  });
};

var LinkGeneric = function LinkGeneric(props) {
  return React.createElement(
    'a',
    {
      key: props.keyP,
      className: props.classes,
      style: props.styles,
      href: props.href,
      target: props.target,
      onClick: function onClick() {
        triggerCustomEvent(props.cta);
      },
      rel: props.rel
    },
    props.label
  );
};

LinkGeneric.propTypes = {
  keyP: PropTypes.string,
  classes: PropTypes.string,
  styles: PropTypes.object,
  label: PropTypes.string,
  href: PropTypes.string,
  rel: PropTypes.string,
  target: PropTypes.string,
  cta: PropTypes.string
};

LinkGeneric.defaultProps = {
  keyP: 'defaultLink',
  classes: '',
  styles: {},
  label: 'Default Link',
  target: '',
  href: 'javascript:void(0)',
  cta: 'CUSTOM_NO_EVENT',
  rel: ''
};

var ButtonGeneric = function ButtonGeneric(props) {
  return React.createElement(
    'button',
    {
      key: props.keyP,
      className: props.classes,
      style: props.styles,
      type: 'button',
      onClick: function onClick() {
        triggerCustomEvent(props.cta);
      }
    },
    props.label
  );
};

ButtonGeneric.propTypes = {
  keyP: PropTypes.string,
  classes: PropTypes.string,
  styles: PropTypes.object,
  label: PropTypes.string,
  cta: PropTypes.string
};

ButtonGeneric.defaultProps = {
  keyP: 'defaultButton',
  classes: '',
  styles: {},
  label: 'Default button',
  cta: 'CUSTOM_NO_EVENT'
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var ctaMapper = {
  link: LinkGeneric,
  button: ButtonGeneric
};

var DivGeneric = function (_Component) {
  inherits(DivGeneric, _Component);

  function DivGeneric() {
    classCallCheck(this, DivGeneric);
    return possibleConstructorReturn(this, (DivGeneric.__proto__ || Object.getPrototypeOf(DivGeneric)).apply(this, arguments));
  }

  createClass(DivGeneric, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      if (this.props.triggerOnMount) {
        var event = new Event(this.props.triggerOnMount);
        window.dispatchEvent(event);
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      if (this.props.triggerOnUnMount) {
        var event = new Event(this.props.triggerOnUnMount);
        window.dispatchEvent(event);
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var props = this.props;

      if (props.beforeElem) {
        var elm = document.getElementById(props.beforeElem);
        if (elm) {
          elm.style.marginTop = '0px';
          var elmTop = elm.offsetTop;
          props.styles.top = elmTop + 10 + 'px';
          props.styles.display = 'block';
          props.styles.position = 'absolute';
          props.styles.paddingLeft = props.divPadLeft + 'px';
          props.divHeight && (elm.style.marginTop = props.divHeight + 'px');
          props.divWidth && (elm.style.marginLeft = props.divWidth + 'px');
        }
      }

      return React.createElement(
        'div',
        { key: props.keyP, className: props.classes, style: props.styles },
        props.heading && React.createElement(
          'p',
          {
            className: props.headingClasses
          },
          props.heading
        ),
        props.content && React.createElement(
          'div',
          {
            className: props.contentClasses
          },
          props.content
        ),
        props.contentHtml && React.createElement('div', { dangerouslySetInnerHTML: { __html: props.contentHtml } }),
        props.ctaList && props.ctaList.map(function (cta) {
          var CtaComp = ctaMapper[cta.type];
          return React.createElement(CtaComp, _extends({}, cta, { key: cta.keyP }));
        })
      );
    }
  }]);
  return DivGeneric;
}(Component);


DivGeneric.propTypes = {
  keyP: PropTypes.string,
  classes: PropTypes.string,
  styles: PropTypes.object,
  beforeElem: PropTypes.string,
  divHeight: PropTypes.number,
  divWidth: PropTypes.number,
  divPadLeft: PropTypes.number,
  heading: PropTypes.string,
  headingClasses: PropTypes.string,
  content: PropTypes.string,
  contentClasses: PropTypes.string,
  contentHtml: PropTypes.string,
  ctaList: PropTypes.array,
  triggerOnMount: PropTypes.string,
  triggerOnUnMount: PropTypes.string
};

DivGeneric.defaultProps = {
  keyP: 'defaultDiv',
  classes: '',
  headingClasses: '',
  contentClasses: '',
  styles: { bottom: 0, position: 'absolute' },
  heading: null,
  content: null,
  contentHtml: null,
  ctaList: [],
  beforeElem: null,
  divHeight: null,
  divWidth: null,
  divPadLeft: null,
  triggerOnMount: null,
  triggerOnUnMount: null
};

var ModalGeneric = function (_Component) {
  inherits(ModalGeneric, _Component);

  function ModalGeneric(props) {
    classCallCheck(this, ModalGeneric);

    var _this = possibleConstructorReturn(this, (ModalGeneric.__proto__ || Object.getPrototypeOf(ModalGeneric)).call(this, props));

    _this.state = {
      isOpen: false
    };
    _this.openModal = _this.openModal.bind(_this);
    _this.closeModal = _this.closeModal.bind(_this);
    _this.bindOpenTriggerEvent = _this.bindOpenTriggerEvent.bind(_this);
    _this.bindCloseTriggerEvent = _this.bindCloseTriggerEvent.bind(_this);
    _this.afterOpenModal = _this.afterOpenModal.bind(_this);
    if (props.openTriggerEvent) {
      _this.bindOpenTriggerEvent(props.openTriggerEvent);
    }
    if (props.closeTriggerEvent) {
      _this.bindCloseTriggerEvent(props.closeTriggerEvent);
    }
    return _this;
  }

  createClass(ModalGeneric, [{
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      if (!this.state.isOpen && nextProps.isOpen) {
        this.openModal();
      }
      if (nextProps.closeModal) {
        this.closeModal();
      }
      if (this.props.triggerOnMount) {
        triggerCustomEvent(this.props.triggerOnMount);
      }
    }
  }, {
    key: 'openModal',
    value: function openModal() {
      this.setState({
        isOpen: true
      });
    }
  }, {
    key: 'closeModal',
    value: function closeModal() {
      var _this2 = this;

      this.setState({
        isOpen: false
      }, function () {
        if (_this2.props.triggerOnClose) {
          triggerCustomEvent(_this2.props.triggerOnClose);
        }
      });
      document.body.style.overflow = '';
    }
  }, {
    key: 'afterOpenModal',
    value: function afterOpenModal() {
      if (this.props.triggerOnOpen) {
        triggerCustomEvent(this.props.triggerOnOpen);
      }
      document.body.style.overflow = 'hidden';
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      if (this.props.triggerOnMount) {
        triggerCustomEvent(this.props.triggerOnMount);
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.closeModal();
    }
  }, {
    key: 'bindOpenTriggerEvent',
    value: function bindOpenTriggerEvent(openTriggerEventName) {
      var _this3 = this;

      window.addEventListener(openTriggerEventName, function () {
        _this3.openModal();
      });
    }
  }, {
    key: 'bindCloseTriggerEvent',
    value: function bindCloseTriggerEvent(closeTriggerEventName) {
      var _this4 = this;

      window.addEventListener(closeTriggerEventName, function () {
        _this4.closeModal();
      });
    }
  }, {
    key: 'render',
    value: function render() {
      return React.createElement(
        Modal,
        {
          isOpen: this.state.isOpen,
          onAfterOpen: this.afterOpenModal,
          onRequestClose: this.closeModal,
          contentLabel: 'Generic Modal',
          ariaHideApp: false,
          style: this.props.styles
        },
        React.createElement(
          'div',
          { style: this.props.closeStyle, onClick: this.closeModal },
          'X'
        ),
        this.props.content && React.createElement(DivGeneric, this.props.content)
      );
    }
  }]);
  return ModalGeneric;
}(Component);

ModalGeneric.propTypes = {
  keyP: PropTypes.string,
  classes: PropTypes.string,
  styles: PropTypes.object,
  closeStyle: PropTypes.object,
  content: PropTypes.object,
  closeModal: PropTypes.bool,
  isOpen: PropTypes.bool,
  openTriggerEvent: PropTypes.string,
  closeTriggerEvent: PropTypes.string,
  triggerOnOpen: PropTypes.string,
  triggerOnClose: PropTypes.string,
  triggerOnMount: PropTypes.string
};
ModalGeneric.defaultProps = {
  keyP: 'defaultModal',
  classes: '',
  styles: {},
  closeStyle: {
    position: 'absolute',
    right: '-25px',
    width: '24px',
    zIndex: 2,
    paddingLeft: '5px',
    overflow: 'hidden',
    background: '#dc3945',
    color: 'white',
    fontWeight: 700
  },
  content: null,
  isOpen: false,
  openTriggerEvent: null,
  closeTriggerEvent: null,
  triggerOnOpen: null,
  triggerOnClose: null,
  triggerOnMount: null
};

var typeCompMapper = {
  button: ButtonGeneric,
  link: LinkGeneric,
  div: DivGeneric,
  modal: ModalGeneric
};

var GenericComponent = function (_Component) {
  inherits(GenericComponent, _Component);

  function GenericComponent(props) {
    classCallCheck(this, GenericComponent);

    var _this = possibleConstructorReturn(this, (GenericComponent.__proto__ || Object.getPrototypeOf(GenericComponent)).call(this, props));

    _this.state = {
      layoutKey: 'default',
      show: false,
      dynamincData: null
    };
    _this.updateStateByVWO = _this.updateStateByVWO.bind(_this);
    return _this;
  }

  createClass(GenericComponent, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      window.addEventListener(this.props.stateUpdateEvent, this.updateStateByVWO);
      if (this.state.dynamincData && this.state.dynamincData.triggerOnMount) {
        var event = new Event(this.state.dynamincData.triggerOnMount);
        window.dispatchEvent(event);
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      window.removeEventListener(this.props.stateUpdateEvent, this.updateStateByVWO);
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      if (this.state.dynamincData && this.state.dynamincData.triggerOnMount) {
        var event = new Event(this.state.dynamincData.triggerOnMount);
        window.dispatchEvent(event);
      }
    }
  }, {
    key: 'updateStateByVWO',
    value: function updateStateByVWO(event) {
      event.detail && this.setState(event.detail);
    }
  }, {
    key: 'render',
    value: function render() {
      if (this.state.show) {
        var overriderObj = _extends({}, this.state.dynamincData);
        return React.createElement(
          'div',
          { key: 'genericComponent' },
          overriderObj.comps && overriderObj.comps.map(function (comp) {
            var Component$$1 = typeCompMapper[comp.type];
            if (!Component$$1) return; // To avoid app crash if unknown type is passed
            return React.createElement(Component$$1, _extends({}, comp, { key: comp.keyP + '-parent' }));
          })
        );
      }
      return null;
    }
  }]);
  return GenericComponent;
}(Component);

GenericComponent.propTypes = {
  stateUpdateEvent: PropTypes$1.string
};

var DynamicReactAB = function (_Component) {
  inherits(DynamicReactAB, _Component);

  function DynamicReactAB() {
    classCallCheck(this, DynamicReactAB);

    var _this = possibleConstructorReturn(this, (DynamicReactAB.__proto__ || Object.getPrototypeOf(DynamicReactAB)).call(this));

    var initArr = typeof window !== 'undefined' && window.dynamicReactDataLayer ? window.dynamicReactDataLayer : [];
    _this.state = {
      experiments: unique(initArr)
    };
    return _this;
  }

  createClass(DynamicReactAB, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      window.dynamicReactDataLayer = window.dynamicReactDataLayer || [];
      var that = this;
      window.dynamicReactDataLayer.push = function (experiment) {
        that.setState({
          experiments: unique([].concat(toConsumableArray(window.dynamicReactDataLayer), [experiment]))
        });
        return Array.prototype.push.apply(this, arguments);
      };
      function initFunc() {
        window.dynamicReactABInit = true;
      }
      window.removeEventListener('DYNAMIC_REACT_AB_MOUNTED', initFunc);
      window.addEventListener('DYNAMIC_REACT_AB_MOUNTED', initFunc);
      window.dispatchEvent(new Event('DYNAMIC_REACT_AB_MOUNTED'));
    }
  }, {
    key: 'render',
    value: function render() {
      var experiments = this.state.experiments;
      console.log('experiments: ', experiments.length);
      return React.createElement(
        'div',
        null,
        experiments.map(function (Experiment) {
          return React.createElement(GenericComponent, { key: Experiment, stateUpdateEvent: Experiment });
        })
      );
    }
  }]);
  return DynamicReactAB;
}(Component);

export default DynamicReactAB;
//# sourceMappingURL=index.es.js.map
